#<profile4xi profile = "10" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "deliver_challenge4xi.py">
#<abstract4xi abstract = "provides the function deliver_challenge4xi"/>
#<refer4xi reference = "connection.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------
#
# -------------------------------------------------------------------------------------------------
#                               

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'deliver_challenge4xi' ]
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import errno
import io
import itertools
import os
import pickle
import select
import socket
import struct
import sys
import time
import tempfile
try:
    import _winapi
    from _winapi import WAIT_OBJECT_0, WAIT_TIMEOUT, INFINITE
except ImportError:
    if sys.platform == 'win32':
        raise
    _winapi = None
    
import hmac     
#"
#/>


# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
from multiprocessing import connection
#"
#/>
 
# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from authentication4xi import AuthenticationError4Xi
from constant_connection4xi import ConstantConnection4Xi
from loop4xi import limit2loop4xi
from print4xi import print4xi
from sleep4xi import omega2sleep4xi
from time4xi import time4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "deliver_challenge" usage = "
#   connection.py   
#       accept(self):
#           if self._authkey:
#               deliver_challenge(c, self._authkey)
#"> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
def deliver_challenge4xi(connection, authkey):
    '''
    deliver_challenge4xi function delivers with (connection, authkey) 
    
    '''
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/6)          
    # -----------------------------------------------------------------------------------------
    
    class2xi = "deliver_challenge4xi"
    method2xi = "deliver_challenge4xi"
    
    message2print = "{(Step 1/6a) started" + "; deliver-challenge4xi-started: " + str(deliver_challenge4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    
    message2print = "{(Step 1/6b) passed" + "; connection-passed-to-deliver-challenge4xi: " + str(connection) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)  
    
    message2print = "{(Step 1/6c) passed" + "; authkey-passed-to-deliver-challenge4xi: " + str(authkey) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)  
       

    # -----------------------------------------------------------------------------------------
    # (Step 2/6)           
    # -----------------------------------------------------------------------------------------  
    if authkey is None:
        authkey = os.urandom(32) #dragonized; create one if not given ! 
        
        message2print = "{(Step 2/6a) set" + "; authkey-set-by-deliver-challenge4xi: " + str(authkey)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
            
    if isinstance(authkey, bytes):
        
        message2print = "{(Step 2/6b) accepted" + "; authkey-accepted-by-deliver-challenge4xi: " + "authkey" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
    else:
        authkey = os.urandom(32) #dragonized; create proper one  !   
        
        message2print = "{(Step 2/8c) set" + "; authkey-set-by-deliver-challenge4xi: " + str(authkey)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
    # -----------------------------------------------------------------------------------------
    # 
    # -----------------------------------------------------------------------------------------  
    ack2connection = False
    nack2connection = True 
                 
    if connection is None:
        message2print = "{(Step 2/8d) warned" + "; connection-warned-by-deliver-challenge4xi: " + " ** " + str(connection)  + " ** " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
    else:
        nack2connection = False
        ack2connection = True           
            
        
    if ack2connection is True:  
        # -----------------------------------------------------------------------------------------
        # (Step 3/6)           
        # -----------------------------------------------------------------------------------------
        cc2xi = ConstantConnection4Xi()
        
        message2print = "{(Step 3/6a) got" + "; cc2xi-got-by-deliver-challenge4xi: " + str(cc2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                        
    
        message2xi = os.urandom(cc2xi.get_message_length())                    # 20 random bytes test message
        
        message2print = "{(Step 3/6b) set" + "; message2xi-set-by-deliver-challenge4xi to: " + str(message2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
                
        message2print = "{(Step 3/6c) passing" + "; challenge-message-passing-to-connection-send_bytes-by-deliver-challenge4xi: " + str(cc2xi.get_challenge() + message2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
                
        ack2send = False
        nack2send = True
        
        count2try = 0
        
        max2count = limit2loop4xi(class2print, method2print) 
        message2print = "{(Step 3/6d) got" + "; max2count-got-by-deliver-challenge4xi: " + str(max2count)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        
        while ((count2try < max2count) and (nack2send is True)):
            # -------------------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------------------     
                        
            try:
       
                # ---------------------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------------------     
                
                connection.send_bytes(cc2xi.get_challenge() + message2xi)                              # b'#CHALLENGE#' + 20 random bytes
            
                message2print = "{(Step 3/6e) performed" + "; connection-send-bytes-performed-by-deliver-challenge4xi: " + str(connection.send_bytes) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                
                ack2send = True
                nack2send = False
            # -------------------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------------------     
                        
            except:
       
                # ---------------------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------------------     
                    
                message2print = "{(Step 3/6f) yielded" + "; nack2send-yielded-by-deliver-challenge4xi: " + str(nack2send)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                
                count2try = count2try + 1
                
                omega2sleep4xi(class2print, method2print)                                              #dragonized giving time to others        
                     
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/6)           
        # -----------------------------------------------------------------------------------------    
        digest2xi = hmac.new(authkey, message2xi).digest()
        
        message2print = "{(Step 4/6a) set" + "; digest2xi-set-by-deliver-challenge4xi to: " + str(digest2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        message2print = "{(Step 4/6b) passing" + "; max-response-message-length-passing-to-connection-recv-bytes-by-deliver-challenge4xi: " + str(cc2xi.get_max_response_message_length()) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
                
                
        ack2response = False
        nack2response = True
        
        count2try = 0
        
        while ((count2try < max2count) and (nack2response is True)):
            # -------------------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------------------     
                    
            try:
       
                # ---------------------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------------------     
                
                response2xi = connection.recv_bytes(cc2xi.get_max_response_message_length())           # reject large message
    
                message2print = "{(Step 4/6c) got" + "; response2xi-got-by-deliver-challenge4xi: " + str(response2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                ack2response = True        
                nack2response = False
            # -------------------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------------------     
                        
            except:
       
                # ---------------------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------------------     
                
                message2print = "{(Step 4/6d) yielded" + "; nack2response-yielded-by-deliver-challenge4xi: " + str(nack2response)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
          
                count2try = count2try + 1
                
                omega2sleep4xi(class2print, method2print)                                              #dragonized giving time to others
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/6)           
        # -----------------------------------------------------------------------------------------
        if ack2response is True:
            # -------------------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------------------     
                
            if response2xi == digest2xi:
       
                # ---------------------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------------------     
                
                ack2send = False
                nack2send = True
        
                count2try = 0
       
                # ---------------------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------------------     
                max2count = limit2loop4xi(class2print, method2print) 
                message2print = "{(Step 5/6a) got" + "; max2count-got-by-deliver-challenge4xi: " + str(max2count)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)        
                
                while ((count2try < max2count) and (nack2send is True)):
    
                    # -----------------------------------------------------------------------------------------    
                    #
                    # -----------------------------------------------------------------------------------------     
                
                    try: 
        
                        # -------------------------------------------------------------------------------------    
                        #
                        # ------------------------------------------------------------------------------------- 
                               
                        message2print = "{(Step 5/6b) passing" + "; welcome-passing-to-connection-send-bytes-by-deliver-challenge4xi: " + str(cc2xi.get_welcome()) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)          
        
                        connection.send_bytes(cc2xi.get_welcome())
        
                        message2print = "{(Step 5/6c) performed" + "; connection-send-bytes-performed-by-deliver-challenge4xi: " + str(connection.send_bytes) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        
                        nack2send = False
                        ack2send = True
                    except:
        
                        # -------------------------------------------------------------------------------------    
                        #
                        # ------------------------------------------------------------------------------------- 
                    
                        message2print = "{(Step 5/6d) yielded" + "; nack2send-yielded-by-deliver-challenge4xi: " + str(nack2send)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
          
                        count2try = count2try + 1
                
                        omega2sleep4xi(class2print, method2print)                                              #dragonized giving time to others                                  
            # -------------------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------------------     
                                
            else:
       
                # ---------------------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------------------     
                
                ack2send = False
                nack2send = True
        
                count2try = 0
       
                # ---------------------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------------------     
            
                while ((count2try < max2count) and (nack2send is True)):
    
                    # -----------------------------------------------------------------------------------------    
                    #
                    # -----------------------------------------------------------------------------------------     
                
                    try: 
        
                        # -------------------------------------------------------------------------------------    
                        #
                        # ------------------------------------------------------------------------------------- 
                                   
                        message2print = "{(Step 5/6d) passing" + "; failure-passing-to-connection-send-bytes-by-deliver-challenge4xi: " + str(cc2xi.get_failure()) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)          
            
            
                        connection.send_bytes(cc2xi.get_failure())
        
        
                        message2print = "{(Step 5/6e) performed" + "; connection-send-bytes-performed-by-deliver-challenge4xi: " + str(connection.send_bytes) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        
                        message2print = "{(Step 5/6f) yielded" + "; digest-yielded-by-deliver-challenge4xi: " + " *** digest received was wrong *** "  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                    except:
        
                        # -------------------------------------------------------------------------------------    
                        #
                        # ------------------------------------------------------------------------------------- 
                    
                        message2print = "{(Step 5/6g) yielded" + "; nack2send-yielded-by-deliver-challenge4xi: " + str(nack2send)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
          
                        count2try = count2try + 1
                
                        omega2sleep4xi(class2print, method2print)                                              #dragonized giving time to others 
       
                # ---------------------------------------------------------------------------------------------    
                # end of while
                # ---------------------------------------------------------------------------------------------     
                                               
            # -------------------------------------------------------------------------------------------------    
            # end of ack2response is True
            # ------------------------------------------------------------------------------------------------- 
                
        # -------------------------------------------------------------------------------------
        #      
        # -------------------------------------------------------------------------------------                              
        else:
            message2print = "{(Step 5/6h) yielded" + "; nack2response-yielded-by-deliver-challenge4xi: " + str(nack2response)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
    # -----------------------------------------------------------------------------------------
    #      
    # -----------------------------------------------------------------------------------------             
    else:
        # -------------------------------------------------------------------------------------
        #      
        # -------------------------------------------------------------------------------------     
        message2print = "{Step 2/8e) yielded" + "; nack2connection-yielded-by-deliver-challenge4xi: " + str(nack2connection)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 6/6)           
    # -----------------------------------------------------------------------------------------
    message2print = "{Step 6/6) returning" + "; deliver-challenge4xi-returning: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
            
    
#</function4xi> 
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of deliver_challenge4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>



